--[[	PhantomPlates Personal NamePlate Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Constants	]]
--------------------------
local DefaultYPos=-128;
local PowerRestState={
--	<not defined> = Rest at max
	[Enum.PowerType.Rage]=0;
	[Enum.PowerType.ComboPoints]=0;
	[Enum.PowerType.RunicPower]=0;
	[Enum.PowerType.SoulShards]=3;
	[Enum.PowerType.LunarPower]=0;
	[Enum.PowerType.HolyPower]=0;
	[Enum.PowerType.Maelstrom]=0;
	[Enum.PowerType.Chi]=0;
	[Enum.PowerType.Insanity]=0;
	[Enum.PowerType.ArcaneCharges]=0;
	[Enum.PowerType.Fury]=0;
	[Enum.PowerType.Pain]=0;
};

----------------------------------
--[[	CUF NamePlate Options	]]
----------------------------------
local PersonalNamePlateFrameOptions=setmetatable({
--	colorHealthBySelection=false;
	displayAggroHighlight=true;
	displayName=false;
	displayStatusText=true;
	healthText="none";

--[[	DefaultCompactNamePlatePlayerFrameOptions from Modern WoW
	displaySelectionHighlight=false;
--	displayAggroHighlight=false;
	displayName=false;
	fadeOutOfRange=false;
	colorNameBySelection=true;
	smoothHealthUpdates=false;
--	displayNameWhenSelected=false;
	showLevel=true;

	healthBarColorOverride=CreateColor(0,1,0);
	defaultBorderColor=CreateColor(0,0,0,1);
--]]
},{__index=DefaultCompactNamePlateFriendlyFrameOptions});

----------------------------------
--[[	Personal NamePlate	]]
----------------------------------
local PersonalNamePlate_Initialized=false;

local PersonalNamePlate=CreateFrame("Frame","PhantomPlates_PersonalNamePlate");
PersonalNamePlate:SetPoint("CENTER",0,DefaultYPos);
PersonalNamePlate:SetSize(128,32);
PersonalNamePlate:Hide();

AddOn.PersonalNamePlate=PersonalNamePlate;

PersonalNamePlate:RegisterEvent("PLAYER_LOGIN");

--	Visibility Events (Basic)
PersonalNamePlate:RegisterEvent("PLAYER_ENTERING_WORLD");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_ENABLED");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_DISABLED");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXHEALTH","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH_FREQUENT","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_SHOW","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_HIDE","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_DISPLAYPOWER","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXPOWER","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_UPDATE","player","pet");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_FREQUENT","player","pet");

if AddOn.ClientVersionMajor>=2 then
--	Visibility Events (Spellcasting)
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_INTERRUPTED","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_DELAYED","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_UPDATE","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_STOP","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_START","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_STOP","player","pet");
	PersonalNamePlate:RegisterUnitEvent("UNIT_SPELLCAST_FAILED","player","pet");
end

if AddOn.ClientVersionMajor>=3 then
--	Visibility Events (Barber Shop)
	PersonalNamePlate:RegisterEvent("BARBER_SHOP_OPEN");
	PersonalNamePlate:RegisterEvent("BARBER_SHOP_CLOSE");
end

--	Posession/Vehicle Event(s)
local UnitSwapEvents={
	"UNIT_PET";
	"UNIT_ENTERING_VEHICLE";
	"UNIT_ENTERED_VEHICLE";
	"UNIT_EXITING_VEHICLE";
	"UNIT_EXITED_VEHICLE";
};
for _,event in ipairs(UnitSwapEvents) do
	UnitSwapEvents[event]=true;
	if event:find("^UNIT_") then
		PersonalNamePlate:RegisterUnitEvent(event,"player");
	else
		PersonalNamePlate:RegisterUnitEvent(event);
	end
end

local function PersonalNamePlate_GetUnit() return UnitIsPossessed("pet") and "pet" or "player"; end--	UnitIsPossessed() implies UnitExists()

local function PersonalNamePlate_IsCasting()
	return	(UnitCastingInfo("player") or UnitChannelInfo("player"))
	or	(UnitIsPossessed("pet") and ((UnitCastingInfo("pet") or UnitChannelInfo("pet"))))
	and true or false;
end

local function PersonalNamePlate_UpdateVisibility(show)
	if not PersonalNamePlate_Initialized then return; end--	Don't run if we haven't loaded yet (ex. Options callback via ADDON_LOADED fires before PLAYER_LOGIN)

--	Visibility Logic
	local unit=PersonalNamePlate_GetUnit();
	local showbypower=false; do
		local powertype,powertoken=UnitPowerType(unit);
		showbypower=UnitPower(unit,powertype)~=(PowerRestState[powertype] or UnitPowerMax(unit,powertype));
	end

	show=(	AddOn.Options.PersonalNamePlate_Enabled--	Enabled?
	and	UIParent:IsVisible()--	UI Shown?
	and	not (BarberShopFrame and BarberShopFrame:IsShown())--	Not in Barber Shop?
	and	(
			AddOn.Options.PersonalNamePlate_AlwaysShow--	Always Show?
		or	(
				AddOn.OptionsPanel:IsVisible()--	Options Panel visible?
			or	(show and true)--	Force shown? (Cast to true; nil is handled below)
			or	(
					show==nil--	Omitted arg? (Preserves false value from above)
				and	(
						InCombatLockdown()--	Show in combat
					or	UnitHealth(unit)<UnitHealthMax(unit)--	Show if health missing
					or	(AddOn.Options.PowerBars_Enabled and showbypower)--	Show if power not at rest state (Only if PowerBars enabled)
					or	(
							AddOn.ClientVersionMajor>=2 and AddOn.Options.PersonalNamePlate_EnableCastBar--	CastBar option TBC+
						and	PersonalNamePlate_IsCasting()--	Show if CastBar shown
						)
					)
				)
			)
		)
	);

--	Apply if changed
	if show~=PersonalNamePlate:IsShown() then
		PersonalNamePlate:SetShown(show);
	end
end

local function PersonalNamePlate_PetCastBar_AdjustPosition()
	PersonalNamePlate.UnitFrame.PetCastBar:ClearAllPoints();
	PersonalNamePlate.UnitFrame.PetCastBar:SetPoint(
		"TOP",PersonalNamePlate.UnitFrame.healthBar,"BOTTOM"
		,17,(PersonalNamePlate.UnitFrame.CastBar:IsShown() and -29 or -9)
	);
end

PersonalNamePlate:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_LOGIN" then--	PersonalNamePlate Init (allow addons hooking NamePlateDriverFrame:OnNamePlateCreated() to work on this)
		NamePlateDriverFrame:OnNamePlateCreated(self);--	Injects NamePlateBaseMixin
		do--	We want our UnitFrame, but asking the frame pool to create one for us spreads taint, so we hack the frame pool to create, but not store it
--			NamePlateDriverFrame:AcquireUnitFrame(self);--	Original call for this (taints frame pool apparently)

			self.UnitFrame=NamePlateDriverFrame.pools:GetPool("NamePlateUnitFrameTemplate"):creationFunc();
			self.UnitFrame:SetParent(self);
			self.UnitFrame:SetPoint("TOPLEFT",self,"TOPLEFT");
			self.UnitFrame:EnableMouse(false);
			self:SetScript("OnSizeChanged",self.OnSizeChanged);
			self:OnSizeChanged();

			AddOn.OnNamePlateCreated(NamePlateDriverFrame,self)
		end

--		RaidTargetIcon is always hidden on player
		self.UnitFrame.RaidTargetFrame:Hide();
		if PersonalNamePlate.UnitFrame.SoftTargetFrame then PersonalNamePlate.UnitFrame.SoftTargetFrame.Icon:Hide(); end

--		Fix level text layering issue (Don't know why this bug doesn't happen on real NamePlates)
		self.UnitFrame.LevelFrame:SetFrameStrata(self.UnitFrame:GetFrameStrata());
		self.UnitFrame.LevelFrame:SetFrameLevel(self.UnitFrame.healthBar.border:GetFrameLevel()+1);

--		Apply Layout (NamePlateDriverFrame won't overwrite this since it never recieves NAME_PLATE_UNIT_ADDED for "player")
		CompactUnitFrame_SetUpFrame(PersonalNamePlate.UnitFrame,DefaultCompactNamePlateFriendlyFrameSetup);--	Expose creation as a friendly NamePlate
		CompactUnitFrame_SetOptionTable(PersonalNamePlate.UnitFrame,PersonalNamePlateFrameOptions);--	Set our own options table
		CompactUnitFrame_UpdateAll(PersonalNamePlate.UnitFrame);--	Update display (Called by CompactUnitFrame_SetUpFrame(), but not CompactUnitFrame_SetOptionTable())

--[[		Setup Conversion Notes
--		These are in NamePlateDriverFrame:ApplyFrameOptions()
		self:OnOptionsUpdated();--	Defunct in classic
		NamePlateDriverFrame:UpdateInsetsForType("friendly",self);--	This isn't a real NamePlate

		NamePlateDriverFrame:OnUnitAuraUpdate("player");
		NamePlateDriverFrame:OnRaidTargetUpdate();
--]]

--		CastBar setup
		if AddOn.ClientVersionMajor>=2 then
			self.UnitFrame.PetCastBar=CreateFrame("StatusBar",nil,self.UnitFrame,"SmallCastingBarFrameTemplate");
			self.UnitFrame.PetCastBar:SetPoint("TOP",self.UnitFrame.healthBar,"BOTTOM",17,-9);
			self.UnitFrame.PetCastBar:SetSize(102,10);
			self.UnitFrame.PetCastBar:Hide();

			self.UnitFrame.PetCastBar:RegisterUnitEvent("UNIT_PET","player");
			self.UnitFrame.PetCastBar:SetScript("OnEvent",function(self,event,...)
				if event=="UNIT_PET" then
					local show=UnitIsPossessed("pet") and AddOn.Options.PersonalNamePlate_EnableCastBar;
					self.showCastbar=show;
					self:SetShown(show and (self.casting or self.channeling or false));
				else CastingBarFrame_OnEvent(self,event,...); end
				
			end);
			self.UnitFrame.PetCastBar:SetScript("OnShow",Nameplate_CastBar_AdjustPosition);
			CastingBarFrame_SetUnit(self.UnitFrame.PetCastBar,"pet",false,false);

			self.UnitFrame.CastBar:HookScript("OnShow",PersonalNamePlate_PetCastBar_AdjustPosition);
			self.UnitFrame.CastBar:HookScript("OnHide",PersonalNamePlate_PetCastBar_AdjustPosition);
			self.UnitFrame.PetCastBar:HookScript("OnShow",PersonalNamePlate_PetCastBar_AdjustPosition);

			if AddOn.CastBarText then--	Optional module
				AddOn.CastBarText.HookCastBar(self.UnitFrame.PetCastBar);
			end

--			Apply startup options
			self.UnitFrame.CastBar.showCastbar=AddOn.Options.PersonalNamePlate_EnableCastBar;
			self.UnitFrame.PetCastBar.showCastbar=UnitIsPossessed("pet") and AddOn.Options.PersonalNamePlate_EnableCastBar;
		end

--		CUF UNIT_POWER_FREQUENT Support
		if AddOn.ClientVersionMajor>=3 then
			function self.UnitFrame.OnEvent(self,event,...)--	CompactUnitFrame_RegisterEvents() uses this as an override
				if self.unit then
--					CUF doesn't support UNIT_POWER_FREQUENT natively, this converts it to UNIT_POWER_UPDATE
					if event=="UNIT_POWER_FREQUENT" then event="UNIT_POWER_UPDATE"; end
					return CompactUnitFrame_OnEvent(self,event,...);
				end
			end
		end

--		Safe to update visibility now
		PersonalNamePlate_Initialized=true;
	end

	if UnitSwapEvents[event] and self:IsShown() and PersonalNamePlate_GetUnit()~=self.namePlateUnitToken then
--		Refresh unit
		self:GetScript("OnHide")(self);
		self:GetScript("OnShow")(self);
	end

--	Force shown if PLAYER_REGEN_DISABLED is firing, otherwise use automated logic (InCombatLockdown() isn't true yet)
	if not InCombatLockdown() then--	Reduce CPU usage in combat since we're force-shown anyway
		PersonalNamePlate_UpdateVisibility(event=="PLAYER_REGEN_DISABLED" or nil);
	end
end);

PersonalNamePlate:SetScript("OnShow",function(self)
	local unit=PersonalNamePlate_GetUnit();
	self:OnAdded(unit,NamePlateDriverFrame);
	if AddOn.ClientVersionMajor>=2 then
--		:OnAdded() calls CastingBarFrame_SetUnit()
--		Set it back to "player" (and enable tradeskills)
		if self.UnitFrame.CastBar.unit=="player" then--	CastingBarFrame_SetUnit() does nothing if the unit was already set to "player"
--			Note: CompactUnitFrame_SetUnit() also calls CastingBarFrame_SetUnit()
			self.UnitFrame.CastBar.showTradeSkills=AddOn.Options.PersonalNamePlate_ShowTradeskills;
			self.UnitFrame.CastBar.showShield=false;
		else CastingBarFrame_SetUnit(self.UnitFrame.CastBar,"player",true,false); end
	end
	NamePlateDriverFrame:OnUnitAuraUpdate(unit);
end);
PersonalNamePlate:SetScript("OnHide",function(self) return self:OnRemoved(); end);

if AddOn.ClientVersionMajor>=3 then
--	CUF doesn't support UNIT_POWER_FREQUENT natively, register it with CompactUnitFrame_UpdateUnitEvents()
	hooksecurefunc("CompactUnitFrame_UpdateUnitEvents",function(self)
		if PersonalNamePlate_Initialized and self==PersonalNamePlate.UnitFrame then
			local unit,display=self.unit,self.displayedUnit;
			display=unit~=display and display or nil;

			self:RegisterUnitEvent("UNIT_POWER_FREQUENT",unit,display);
		end
	end);
end

local function OptionsPanel_OnVisibilityChanged() return PersonalNamePlate_UpdateVisibility(); end
AddOn.OptionsPanel:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
AddOn.OptionsPanel:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
UIParent:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);

----------------------------------
--[[	Options Defaults	]]
----------------------------------
AddOn.Options.PersonalNamePlate_Enabled=true;
AddOn.Options.PersonalNamePlate_AlwaysShow=false;
AddOn.Options.PersonalNamePlate_EnableCastBar=true;
AddOn.Options.PersonalNamePlate_ShowTradeskills=true;
AddOn.Options.PersonalNamePlate_HideDefaultCastBar=true;
AddOn.Options.PersonalNamePlate_YPos=DefaultYPos;

--------------------------
--[[	Options Layout	]]
--------------------------
--	Note: UICheckButtonTemplate's .text is depreciated in favor of .Text, but Vanilla Classic still uses it

local SubPanel=AddOn.OptionsPanel:CreateSubPanel();
SubPanel:SetHeight(AddOn.ClientVersionMajor<=1 and 104 or 176);

local EnableButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
EnableButton:SetPoint("TOPLEFT",4,0);
(EnableButton.Text or EnableButton.text):SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_Enabled);

local AlwaysShowButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
AlwaysShowButton:SetPoint("TOPLEFT",28,-24);
(AlwaysShowButton.Text or AlwaysShowButton.text):SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_AlwaysShow);

local EnableCastBarButton;
local ShowTradeskillsButton;
local HideDefaultCastBarButton;
if AddOn.ClientVersionMajor>=2 then
	EnableCastBarButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
	EnableCastBarButton:SetPoint("TOPLEFT",28,-48);
	(EnableCastBarButton.Text or EnableCastBarButton.text):SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_EnableCastBar);

	ShowTradeskillsButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
	ShowTradeskillsButton:SetPoint("TOPLEFT",52,-72);
	(ShowTradeskillsButton.Text or ShowTradeskillsButton.text):SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_ShowTradeskills);

	HideDefaultCastBarButton=CreateFrame("CheckButton",nil,SubPanel,"UICheckButtonTemplate");
	HideDefaultCastBarButton:SetPoint("TOPLEFT",52,-96);
	(HideDefaultCastBarButton.Text or HideDefaultCastBarButton.text):SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_HideDefaultCastBar);
end

local YPosSlider=CreateFrame("Slider",nil,SubPanel,"OptionsSliderTemplate");
YPosSlider:SetPoint("TOP",0,AddOn.ClientVersionMajor<=1 and -64 or -136);
YPosSlider:SetWidth(256);

YPosSlider:SetMinMaxValues(-384,384);
YPosSlider:SetValue(DefaultYPos);

YPosSlider:SetValueStep(16);
YPosSlider:SetStepsPerPage(1);
YPosSlider:SetObeyStepOnDrag(true);

YPosSlider.Mid=YPosSlider:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
YPosSlider.Mid:SetPoint("TOP",YPosSlider,"BOTTOM",0,3);

YPosSlider.Text:SetFontObject(GameFontHighlightSmall);
YPosSlider.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,DefaultYPos);
YPosSlider.Low:SetText(-384);
YPosSlider.Mid:SetText(0);
YPosSlider.High:SetText(384);

--------------------------
--[[	Options Setup	]]
--------------------------
local function SetCastBarVisibility(bar,show)
	bar.showCastbar=show;
	bar:SetShown(show and (bar.casting or bar.channeling or false));--	.casting and .channeling are true/nil
end

AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_Enabled",EnableButton
	,nil
	,EnableButton.SetChecked
	,EnableButton.GetChecked
	,"OnClick"
);

AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_AlwaysShow",AlwaysShowButton
	,nil
	,AlwaysShowButton.SetChecked
	,AlwaysShowButton.GetChecked
	,"OnClick"
);

if AddOn.ClientVersionMajor>=2 then
	AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_EnableCastBar",EnableCastBarButton
		,nil
		,EnableCastBarButton.SetChecked
		,EnableCastBarButton.GetChecked
		,"OnClick"
	);

	AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_ShowTradeskills",ShowTradeskillsButton
		,nil
		,ShowTradeskillsButton.SetChecked
		,ShowTradeskillsButton.GetChecked
		,"OnClick"
	);

	AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_HideDefaultCastBar",HideDefaultCastBarButton
		,nil
		,HideDefaultCastBarButton.SetChecked
		,HideDefaultCastBarButton.GetChecked
		,"OnClick"
	);
end

AddOn.OptionsPanel:RegisterControl("PersonalNamePlate_YPos",YPosSlider
	,nil
	,YPosSlider.SetValue
	,YPosSlider.GetValue
	,"OnValueChanged"
);

AddOn.OptionsPanel:HookOption("PersonalNamePlate_Enabled",function(_,_,val)
	local control_alwaysshow=AddOn.OptionsPanel:GetControl("PersonalNamePlate_AlwaysShow");
	control_alwaysshow:SetEnabled(val);
	(control_alwaysshow.Text or control_alwaysshow.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

	if AddOn.ClientVersionMajor>=2 then
		local control_enablecastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_EnableCastBar");
		control_enablecastbar:SetEnabled(val);
		(control_enablecastbar.Text or control_enablecastbar.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

		do	local val=val and AddOn.Options.PersonalNamePlate_EnableCastBar;
			local control_showtradeskills=AddOn.OptionsPanel:GetControl("PersonalNamePlate_ShowTradeskills");
			control_showtradeskills:SetEnabled(val);
			(control_showtradeskills.Text or control_showtradeskills.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

			local control_hidedefaultcastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_HideDefaultCastBar");
			control_hidedefaultcastbar:SetEnabled(val);
			(control_hidedefaultcastbar.Text or control_hidedefaultcastbar.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);
		end

		local show=not (val and AddOn.Options.PersonalNamePlate_EnableCastBar and AddOn.Options.PersonalNamePlate_HideDefaultCastBar);
		SetCastBarVisibility(CastingBarFrame,show);
		SetCastBarVisibility(PetCastingBarFrame,UnitIsPossessed("pet") and show);
	end

	local control_ypos=AddOn.OptionsPanel:GetControl("PersonalNamePlate_YPos");
	control_ypos:SetEnabled(val);
	do	local fontobj=val and GameFontHighlightSmall or GameFontDisableSmall;
		control_ypos.Text:SetFontObject(fontobj);
		control_ypos.Low:SetFontObject(fontobj);
		control_ypos.Mid:SetFontObject(fontobj);
		control_ypos.High:SetFontObject(fontobj);
	end

	PersonalNamePlate_UpdateVisibility();
end);

AddOn.OptionsPanel:HookOption("PersonalNamePlate_AlwaysShow",function()
	PersonalNamePlate_UpdateVisibility();--	Directly setting this as the callback will cause it to recieve the control as the optional force show flag
end);

if AddOn.ClientVersionMajor>=2 then
	AddOn.OptionsPanel:HookOption("PersonalNamePlate_EnableCastBar",function(_,_,val)
		do	local val=val and AddOn.Options.PersonalNamePlate_Enabled;
			local control_showtradeskills=AddOn.OptionsPanel:GetControl("PersonalNamePlate_ShowTradeskills");
			control_showtradeskills:SetEnabled(val);
			(control_showtradeskills.Text or control_showtradeskills.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);

			local control_hidedefaultcastbar=AddOn.OptionsPanel:GetControl("PersonalNamePlate_HideDefaultCastBar");
			control_hidedefaultcastbar:SetEnabled(val);
			(control_hidedefaultcastbar.Text or control_hidedefaultcastbar.text):SetFontObject(val and GameFontNormalSmall or GameFontDisableSmall);
		end

		local show=not (AddOn.Options.PersonalNamePlate_Enabled and val and AddOn.Options.PersonalNamePlate_HideDefaultCastBar);
		SetCastBarVisibility(CastingBarFrame,show);
		SetCastBarVisibility(PetCastingBarFrame,UnitIsPossessed("pet") and show);

		if PersonalNamePlate_Initialized then
			SetCastBarVisibility(PersonalNamePlate.UnitFrame.CastBar,val);
			PersonalNamePlate_UpdateVisibility();
		end
	end);

	PetCastingBarFrame:HookScript("OnEvent",function(self,event,...)
		if event=="UNIT_PET" and (...)=="player" then
			SetCastBarVisibility(self,UnitIsPossessed("pet") and not AddOn.Options.PersonalNamePlate_HideDefaultCastBar);
		end
	end);

	AddOn.OptionsPanel:HookOption("PersonalNamePlate_ShowTradeskills",function(_,_,val)
		if not PersonalNamePlate_Initialized then return; end
		self.UnitFrame.CastBar.showTradeSkills=val;
	end);

	AddOn.OptionsPanel:HookOption("PersonalNamePlate_HideDefaultCastBar",function(_,_,val)
		local show=not (AddOn.Options.PersonalNamePlate_Enabled and AddOn.Options.PersonalNamePlate_EnableCastBar and val);
		SetCastBarVisibility(CastingBarFrame,show);
		SetCastBarVisibility(PetCastingBarFrame,UnitIsPossessed("pet") and show);
	end);
end

AddOn.OptionsPanel:HookOption("PersonalNamePlate_YPos",function(self,_,val)
	self.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,val);
	PersonalNamePlate:SetPoint("CENTER",0,val);
end);

AddOn.OptionsPanel:HookOption("PowerBars_Enabled",function()
	if AddOn.Options.PersonalNamePlate_Enabled then PersonalNamePlate_UpdateVisibility(); end
end);
